@@ -65,6 +65,7 @@ static LPCWSTR      PRSRV_JDK    = L"jdk";
 static LPCWSTR      PRSRV_JRE    = L"jre";
 static LPCWSTR      PRSRV_MANUAL = L"manual";
 static LPCWSTR      PRSRV_JBIN   = L"\\bin\\java.exe";
+static LPCWSTR      PRSRV_PBIN   = L"\\bin";
 static LPCWSTR      PRSRV_SIGNAL = L"SIGNAL";
 
 static LPCWSTR      STYPE_INTERACTIVE = L"interactive";
@@ -98,37 +99,38 @@ static APXCMDLINEOPT _options[] = {
 /* 8  */    { L"Environment",       L"Environment",     NULL,           APXCMDOPT_MSZ | APXCMDOPT_REG, NULL, 0},
 /* 9  */    { L"User",              L"User",            NULL,           APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
 /* 10 */    { L"Password",          L"Password",        NULL,           APXCMDOPT_BIN | APXCMDOPT_REG, NULL, 0},
+/* 11 */    { L"LibraryPath",       L"LibraryPath",     NULL,           APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
 
-/* 11 */    { L"JavaHome",          L"JavaHome",        L"Java",        APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
-/* 12 */    { L"Jvm",               L"Jvm",             L"Java",        APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
-/* 13 */    { L"JvmOptions",        L"Options",         L"Java",        APXCMDOPT_MSZ | APXCMDOPT_REG, NULL, 0},
-/* 14 */    { L"Classpath",         L"Classpath",       L"Java",        APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
-/* 15 */    { L"JvmMs",             L"JvmMs",           L"Java",        APXCMDOPT_INT | APXCMDOPT_REG, NULL, 0},
-/* 16 */    { L"JvmMx",             L"JvmMx",           L"Java",        APXCMDOPT_INT | APXCMDOPT_REG, NULL, 0},
-/* 17 */    { L"JvmSs",             L"JvmSs",           L"Java",        APXCMDOPT_INT | APXCMDOPT_REG, NULL, 0},
+/* 12 */    { L"JavaHome",          L"JavaHome",        L"Java",        APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
+/* 13 */    { L"Jvm",               L"Jvm",             L"Java",        APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
+/* 14 */    { L"JvmOptions",        L"Options",         L"Java",        APXCMDOPT_MSZ | APXCMDOPT_REG, NULL, 0},
+/* 15 */    { L"Classpath",         L"Classpath",       L"Java",        APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
+/* 16 */    { L"JvmMs",             L"JvmMs",           L"Java",        APXCMDOPT_INT | APXCMDOPT_REG, NULL, 0},
+/* 17 */    { L"JvmMx",             L"JvmMx",           L"Java",        APXCMDOPT_INT | APXCMDOPT_REG, NULL, 0},
+/* 19 */    { L"JvmSs",             L"JvmSs",           L"Java",        APXCMDOPT_INT | APXCMDOPT_REG, NULL, 0},
 
-/* 18 */    { L"StopImage",         L"Image",           L"Stop",        APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
-/* 19 */    { L"StopPath",          L"WorkingPath",     L"Stop",        APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
-/* 20 */    { L"StopClass",         L"Class",           L"Stop",        APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
-/* 21 */    { L"StopParams",        L"Params",          L"Stop",        APXCMDOPT_MSZ | APXCMDOPT_REG, NULL, 0},
-/* 22 */    { L"StopMethod",        L"Method",          L"Stop",        APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
-/* 23 */    { L"StopMode",          L"Mode",            L"Stop",        APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
-/* 24 */    { L"StopTimeout",       L"Timeout",         L"Stop",        APXCMDOPT_INT | APXCMDOPT_REG, NULL, 0},
+/* 19 */    { L"StopImage",         L"Image",           L"Stop",        APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
+/* 20 */    { L"StopPath",          L"WorkingPath",     L"Stop",        APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
+/* 21 */    { L"StopClass",         L"Class",           L"Stop",        APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
+/* 22 */    { L"StopParams",        L"Params",          L"Stop",        APXCMDOPT_MSZ | APXCMDOPT_REG, NULL, 0},
+/* 23 */    { L"StopMethod",        L"Method",          L"Stop",        APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
+/* 24 */    { L"StopMode",          L"Mode",            L"Stop",        APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
+/* 25 */    { L"StopTimeout",       L"Timeout",         L"Stop",        APXCMDOPT_INT | APXCMDOPT_REG, NULL, 0},
 
-/* 25 */    { L"StartImage",        L"Image",           L"Start",       APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
-/* 26 */    { L"StartPath",         L"WorkingPath",     L"Start",       APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
-/* 27 */    { L"StartClass",        L"Class",           L"Start",       APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
-/* 28 */    { L"StartParams",       L"Params",          L"Start",       APXCMDOPT_MSZ | APXCMDOPT_REG, NULL, 0},
-/* 29 */    { L"StartMethod",       L"Method",          L"Start",       APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
-/* 30 */    { L"StartMode",         L"Mode",            L"Start",       APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
+/* 26 */    { L"StartImage",        L"Image",           L"Start",       APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
+/* 27 */    { L"StartPath",         L"WorkingPath",     L"Start",       APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
+/* 28 */    { L"StartClass",        L"Class",           L"Start",       APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
+/* 29 */    { L"StartParams",       L"Params",          L"Start",       APXCMDOPT_MSZ | APXCMDOPT_REG, NULL, 0},
+/* 30 */    { L"StartMethod",       L"Method",          L"Start",       APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
+/* 31 */    { L"StartMode",         L"Mode",            L"Start",       APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
 
-/* 31 */    { L"LogPath",           L"Path",            L"Log",         APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
-/* 32 */    { L"LogPrefix",         L"Prefix",          L"Log",         APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
-/* 33 */    { L"LogLevel",          L"Level",           L"Log",         APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
-/* 34 */    { L"StdError",          L"StdError",        L"Log",         APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
-/* 35 */    { L"StdOutput",         L"StdOutput",       L"Log",         APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
-/* 36 */    { L"LogJniMessages",    L"LogJniMessages",  L"Log",         APXCMDOPT_INT | APXCMDOPT_REG, NULL, 1},
-/* 37 */    { L"PidFile",           L"PidFile",         L"Log",         APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
+/* 32 */    { L"LogPath",           L"Path",            L"Log",         APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
+/* 33 */    { L"LogPrefix",         L"Prefix",          L"Log",         APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
+/* 34 */    { L"LogLevel",          L"Level",           L"Log",         APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
+/* 35 */    { L"StdError",          L"StdError",        L"Log",         APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
+/* 36 */    { L"StdOutput",         L"StdOutput",       L"Log",         APXCMDOPT_STE | APXCMDOPT_REG, NULL, 0},
+/* 37 */    { L"LogJniMessages",    L"LogJniMessages",  L"Log",         APXCMDOPT_INT | APXCMDOPT_REG, NULL, 1},
+/* 38 */    { L"PidFile",           L"PidFile",         L"Log",         APXCMDOPT_STR | APXCMDOPT_REG, NULL, 0},
             /* NULL terminate the array */
             { NULL }
 };
@@ -158,38 +160,39 @@ static APXCMDLINEOPT _options[] = {
 
 #define SO_USER             GET_OPT_V(9)
 #define SO_PASSWORD         GET_OPT_V(10)
+#define SO_LIBPATH          GET_OPT_V(11)
 
-#define SO_JAVAHOME         GET_OPT_V(11)
-#define SO_JVM              GET_OPT_V(12)
-#define SO_JVMOPTIONS       GET_OPT_V(13)
-#define SO_CLASSPATH        GET_OPT_V(14)
-#define SO_JVMMS            GET_OPT_I(15)
-#define SO_JVMMX            GET_OPT_I(16)
-#define SO_JVMSS            GET_OPT_I(17)
+#define SO_JAVAHOME         GET_OPT_V(12)
+#define SO_JVM              GET_OPT_V(13)
+#define SO_JVMOPTIONS       GET_OPT_V(14)
+#define SO_CLASSPATH        GET_OPT_V(15)
+#define SO_JVMMS            GET_OPT_I(16)
+#define SO_JVMMX            GET_OPT_I(17)
+#define SO_JVMSS            GET_OPT_I(18)
 
-#define SO_STOPIMAGE        GET_OPT_V(18)
-#define SO_STOPPATH         GET_OPT_V(19)
-#define SO_STOPCLASS        GET_OPT_V(20)
-#define SO_STOPPARAMS       GET_OPT_V(21)
-#define SO_STOPMETHOD       GET_OPT_V(22)
-#define SO_STOPMODE         GET_OPT_V(23)
-#define SO_STOPTIMEOUT      GET_OPT_I(24)
+#define SO_STOPIMAGE        GET_OPT_V(19)
+#define SO_STOPPATH         GET_OPT_V(20)
+#define SO_STOPCLASS        GET_OPT_V(21)
+#define SO_STOPPARAMS       GET_OPT_V(22)
+#define SO_STOPMETHOD       GET_OPT_V(23)
+#define SO_STOPMODE         GET_OPT_V(24)
+#define SO_STOPTIMEOUT      GET_OPT_I(25)
 
-#define SO_STARTIMAGE       GET_OPT_V(25)
-#define SO_STARTPATH        GET_OPT_V(26)
-#define SO_STARTCLASS       GET_OPT_V(27)
-#define SO_STARTPARAMS      GET_OPT_V(28)
-#define SO_STARTMETHOD      GET_OPT_V(29)
-#define SO_STARTMODE        GET_OPT_V(30)
+#define SO_STARTIMAGE       GET_OPT_V(26)
+#define SO_STARTPATH        GET_OPT_V(27)
+#define SO_STARTCLASS       GET_OPT_V(28)
+#define SO_STARTPARAMS      GET_OPT_V(29)
+#define SO_STARTMETHOD      GET_OPT_V(30)
+#define SO_STARTMODE        GET_OPT_V(31)
 
-#define SO_LOGPATH          GET_OPT_V(31)
-#define SO_LOGPREFIX        GET_OPT_V(32)
-#define SO_LOGLEVEL         GET_OPT_V(33)
+#define SO_LOGPATH          GET_OPT_V(32)
+#define SO_LOGPREFIX        GET_OPT_V(33)
+#define SO_LOGLEVEL         GET_OPT_V(34)
 
-#define SO_STDERROR         GET_OPT_V(34)
-#define SO_STDOUTPUT        GET_OPT_V(35)
-#define SO_JNIVFPRINTF      GET_OPT_I(36)
-#define SO_PIDFILE          GET_OPT_V(37)
+#define SO_STDERROR         GET_OPT_V(35)
+#define SO_STDOUTPUT        GET_OPT_V(36)
+#define SO_JNIVFPRINTF      GET_OPT_I(37)
+#define SO_PIDFILE          GET_OPT_V(38)
 
 /* Main service table entry
  * filled at run-time
@@ -541,7 +544,7 @@ static BOOL docmdInstallService(LPAPXCMDLINE lpCmdline)
         dwType |= SERVICE_INTERACTIVE_PROCESS;
 
     /* Check if --Install is provided */
-    if (!SO_INSTALL) {
+    if (!IS_VALID_STRING(SO_INSTALL)) {
         lstrlcpyW(szImage, SIZ_HUGLEN, lpCmdline->szExePath);
         lstrlcatW(szImage, SIZ_HUGLEN, L"\\");
         lstrlcatW(szImage, SIZ_HUGLEN, lpCmdline->szExecutable);
@@ -830,11 +833,11 @@ static DWORD WINAPI serviceStop(LPVOID lpParameter)
         return TRUE;    /* Nothing to do */
     }
     if (_jni_shutdown) {
-        if (!SO_STARTPATH && SO_STOPPATH) {
+        if (!IS_VALID_STRING(SO_STARTPATH) && IS_VALID_STRING(SO_STOPPATH)) {
             /* If the Working path is specified change the current directory
              * but only if the start path wasn't specified already.
              */
-            SetCurrentDirectoryW(SO_STARTPATH);
+            SetCurrentDirectoryW(SO_STOPPATH);
         }
         hWorker = apxCreateJava(gPool, _jni_jvmpath);
         if (IS_INVALID_HANDLE(hWorker)) {
@@ -853,6 +856,7 @@ static DWORD WINAPI serviceStop(LPVOID lpParameter)
         gSargs.lpArguments      = _jni_sparam;
         gSargs.szStdErrFilename = NULL;
         gSargs.szStdOutFilename = NULL;
+        gSargs.szLibraryPath    = SO_LIBPATH;
 
         if (lstrcmpA(_jni_sclass, "java/lang/System") == 0)
             _onexit(onExitHook);
@@ -874,11 +878,11 @@ static DWORD WINAPI serviceStop(LPVOID lpParameter)
         }
         wait_to_die = TRUE;
     }
-    else if (SO_STOPMODE) { /* Only in case we have a stop mode */
+    else if (IS_VALID_STRING(SO_STOPMODE)) { /* Only in case we have a stop mode */
         DWORD nArgs;
         LPWSTR *pArgs;
 
-        if (!SO_STOPIMAGE) {
+        if (!IS_VALID_STRING(SO_STOPIMAGE)) {
             apxLogWrite(APXLOG_MARK_ERROR "Missing service ImageFile");
             if (!_service_mode)
                 apxDisplayError(FALSE, NULL, 0, "Service '%S' is missing the ImageFile",
@@ -1011,7 +1015,7 @@ static DWORD serviceStart()
         apxLogWrite(APXLOG_MARK_INFO "Worker is not defined");
         return TRUE;    /* Nothing to do */
     }
-    if (SO_PIDFILE) {
+    if (IS_VALID_STRING(SO_PIDFILE)) {
         gPidfileName = apxLogFile(gPool, SO_LOGPATH, SO_PIDFILE, NULL, FALSE);
         if (GetFileAttributesW(gPidfileName) !=  INVALID_FILE_ATTRIBUTES) {
             /* Pid file exists */
@@ -1025,10 +1029,14 @@ static DWORD serviceStart()
     }
     GetSystemTimeAsFileTime(&fts);
     if (_jni_startup) {
-        if (SO_STARTPATH) {
+        if (IS_VALID_STRING(SO_STARTPATH)) {
             /* If the Working path is specified change the current directory */
             SetCurrentDirectoryW(SO_STARTPATH);
         }
+        if (IS_VALID_STRING(SO_LIBPATH)) {
+            /* Add LibraryPath to the PATH */
+           apxAddToPathW(gPool, SO_LIBPATH);
+        }
         /* Set the environment using putenv, so JVM can use it */
         setInprocEnvironment();
         /* Create the JVM glbal worker */
@@ -1049,6 +1057,7 @@ static DWORD serviceStart()
         gRargs.lpArguments      = _jni_rparam;
         gRargs.szStdErrFilename = gStdwrap.szStdErrFilename;
         gRargs.szStdOutFilename = gStdwrap.szStdOutFilename;
+        gRargs.szLibraryPath    = SO_LIBPATH;
 
         if (!apxJavaStart(&gRargs)) {
             rv = 4;
@@ -1058,13 +1067,17 @@ static DWORD serviceStart()
         apxLogWrite(APXLOG_MARK_DEBUG "Java started %s", _jni_rclass);
     }
     else {
-        if (!SO_STARTIMAGE) {
+        if (!IS_VALID_STRING(SO_STARTIMAGE)) {
             apxLogWrite(APXLOG_MARK_ERROR "Missing service ImageFile");
             if (!_service_mode)
                 apxDisplayError(FALSE, NULL, 0, "Service '%S' is missing the ImageFile",
                                 _service_name ? _service_name : L"unknown");
             return 1;
         }
+        if (IS_VALID_STRING(SO_LIBPATH)) {
+            /* Add LibraryPath to the PATH */
+           apxAddToPathW(gPool, SO_LIBPATH);
+        }
         /* Redirect process */
         gWorker = apxCreateProcessW(gPool,
                                     0,
@@ -1245,7 +1258,7 @@ void WINAPI serviceMain(DWORD argc, LPTSTR *argv)
 
     apxLogWrite(APXLOG_MARK_DEBUG "Inside ServiceMain...");
 
-    if (_service_name) {
+    if (IS_VALID_STRING(_service_name)) {
         WCHAR en[SIZ_HUGLEN];
         int i;
         PSECURITY_ATTRIBUTES sa = GetNullACL();
@@ -1265,7 +1278,7 @@ void WINAPI serviceMain(DWORD argc, LPTSTR *argv)
         }
     }
     /* Check the StartMode */
-    if (SO_STARTMODE) {
+    if (IS_VALID_STRING(SO_STARTMODE)) {
         if (!lstrcmpiW(SO_STARTMODE, PRSRV_JVM)) {
             _jni_startup = TRUE;
             _jni_rclass  = WideToUTF8(SO_STARTCLASS);
@@ -1289,7 +1302,13 @@ void WINAPI serviceMain(DWORD argc, LPTSTR *argv)
                 jx = apxPoolAlloc(gPool, (lstrlenW(szJH) + 16) * sizeof(WCHAR));
                 lstrcpyW(jx, szJH);
                 lstrcatW(jx, PRSRV_JBIN);
-                SO_STARTPATH = szJH;
+                if (!SO_STARTPATH) {
+                    /* Use JAVA_HOME/bin as start path */
+                    LPWSTR szJP = apxPoolAlloc(gPool, (lstrlenW(szJH) + 8) * sizeof(WCHAR));
+                    lstrcpyW(szJP, szJH);
+                    lstrcatW(szJP, PRSRV_PBIN);
+                    SO_STARTPATH = szJP;
+                }
             }
             else {
                 apxLogWrite(APXLOG_MARK_ERROR "Unable to find Java Runtime Environment.");
@@ -1301,7 +1320,7 @@ void WINAPI serviceMain(DWORD argc, LPTSTR *argv)
         }
     }
     /* Check the StopMode */
-    if (SO_STOPMODE) {
+    if (IS_VALID_STRING(SO_STOPMODE)) {
         if (!lstrcmpiW(SO_STOPMODE, PRSRV_JVM)) {
             _jni_shutdown = TRUE;
             _jni_sclass = WideToUTF8(SO_STOPCLASS);
@@ -1324,7 +1343,13 @@ void WINAPI serviceMain(DWORD argc, LPTSTR *argv)
                 jx = apxPoolAlloc(gPool, (lstrlenW(szJH) + 16) * sizeof(WCHAR));
                 lstrcpyW(jx, szJH);
                 lstrcatW(jx, PRSRV_JBIN);
-                SO_STOPPATH = szJH;
+                if (!SO_STOPPATH) {
+                    LPWSTR szJP = apxPoolAlloc(gPool, (lstrlenW(szJH) + 8) * sizeof(WCHAR));
+                    lstrcpyW(szJP, szJH);
+                    lstrcatW(szJP, PRSRV_PBIN);
+                    /* Use JAVA_HOME/bin as stop path */
+                    SO_STOPPATH = szJP;
+                }
             }
             else {
                 apxLogWrite(APXLOG_MARK_ERROR "Unable to find Java Runtime Environment.");
@@ -1337,15 +1362,15 @@ void WINAPI serviceMain(DWORD argc, LPTSTR *argv)
     }
     /* Find the classpath */
     if (_jni_shutdown || _jni_startup) {
-        if (SO_JVM) {
+        if (IS_VALID_STRING(SO_JVM)) {
             if (lstrcmpW(SO_JVM, PRSRV_AUTO))
                 _jni_jvmpath = SO_JVM;
         }
-        if (SO_CLASSPATH)
+        if (IS_VALID_STRING(SO_CLASSPATH))
             _jni_classpath = WideToUTF8(SO_CLASSPATH);
-        if (SO_STARTMETHOD)
+        if (IS_VALID_STRING(SO_STARTMETHOD))
             _jni_rmethod   = WideToAscii(SO_STARTMETHOD, (LPSTR)SO_STARTMETHOD);
-        if (SO_STOPMETHOD)
+        if (IS_VALID_STRING(SO_STOPMETHOD))
             _jni_smethod   = WideToAscii(SO_STOPMETHOD, (LPSTR)SO_STOPMETHOD);
         _jni_jvmoptions    = MzWideToUTF8(SO_JVMOPTIONS);
     }
@@ -1508,7 +1533,7 @@ void __cdecl main(int argc, char **argv)
         SYSTEMTIME t;
         GetLocalTime(&t);
         fprintf(stdout, "\n%d-%02d-%02d %02d:%02d:%02d "
-                        "Commons Daemon procrun stdout initialized",
+                        "Commons Daemon procrun stdout initialized\n",
                         t.wYear, t.wMonth, t.wDay,
                         t.wHour, t.wMinute, t.wSecond);
         fprintf(stderr, "\n%d-%02d-%02d %02d:%02d:%02d "
